diff --git a/openbsc/src/libbsc/abis_nm.c b/openbsc/src/libbsc/abis_nm.c
index 6568b53..fd186f7 100644
--- a/openbsc/src/libbsc/abis_nm.c
+++ b/openbsc/src/libbsc/abis_nm.c
@@ -359,7 +359,7 @@ static int ipacc_sw_activate(struct gsm_bts *bts, uint8_t obj_class, uint8_t i0,
 	return abis_nm_sendmsg(bts, msg);
 }
 
-static int abis_nm_parse_sw_descr(const uint8_t *sw_descr, int sw_descr_len)
+static int abis_nm_parse_sw_descr(const uint8_t *sw_descr, int sw_descr_len, int *ver_ofs)
 {
 	static const struct tlv_definition sw_descr_def = {
 		.def = {
@@ -373,6 +373,9 @@ static int abis_nm_parse_sw_descr(const uint8_t *sw_descr, int sw_descr_len)
 	const uint8_t *val;
 	int ofs = 0, len;
 
+	/* Set to "Version not found" initially */
+	*ver_ofs = -1;
+
 	/* Classic TLV parsing doesn't work well with SW_DESCR because of it's
 	 * nested nature and the fact you have to assume it contains only two sub
 	 * tags NM_ATT_FILE_VERSION & NM_ATT_FILE_ID to parse it */
@@ -397,6 +400,7 @@ static int abis_nm_parse_sw_descr(const uint8_t *sw_descr, int sw_descr_len)
 		DEBUGP(DNM, "FILE_VERSION attribute identifier not found!\n");
 		return -3;
 	}
+	*ver_ofs = ofs+4;
 	ofs += len;
 
 	return ofs;
@@ -409,7 +413,8 @@ static int abis_nm_rx_sw_act_req(struct msgb *mb)
 	struct e1inp_sign_link *sign_link = mb->dst;
 	struct tlv_parsed tp;
 	const uint8_t *sw_config;
-	int ret, sw_config_len, sw_descr_len;
+	const uint8_t *sw_config_ptr;
+	int ret, sw_config_len, sw_descr_len_act, sw_descr_len[2], ver_ofs[2];
 
 	abis_nm_debugp_foh(DNM, foh);
 
@@ -439,16 +444,39 @@ static int abis_nm_rx_sw_act_req(struct msgb *mb)
 		DEBUGP(DNM, "Found SW config: %s\n", osmo_hexdump(sw_config, sw_config_len));
 	}
 
-		/* Use the first SW_DESCR present in SW config */
-	sw_descr_len = abis_nm_parse_sw_descr(sw_config, sw_config_len);
-	if (sw_descr_len < 0)
+	/* Get the first SW_DESCR present in SW config */
+	sw_config_ptr = sw_config;
+	sw_descr_len[0] = abis_nm_parse_sw_descr(sw_config_ptr, sw_config_len, &ver_ofs[0]);
+	if (sw_descr_len[0] < 0)
 		return -EINVAL;
+	sw_descr_len_act = sw_descr_len[0];
+	DEBUGP(DNM, "Found first SW version: %s\n", sw_config+ver_ofs[0]);
+
+	/* Get the second SW_DESCR if it is present in SW config */
+	if (sw_config_len <= sw_descr_len[0]) {
+		DEBUGP(DNM, "No second SW found.\n");
+	} else {
+		sw_config_ptr = sw_config+sw_descr_len[0];
+		sw_descr_len[1] = abis_nm_parse_sw_descr(sw_config_ptr, sw_config_len, &ver_ofs[1]);
+		if (sw_descr_len[1] > 0) {
+			DEBUGP(DNM, "Found second SW version: %s\n", sw_config_ptr+ver_ofs[1]);
+
+			/* Select the latest firmware */
+			if (strncmp((const char*)(sw_config+ver_ofs[0]), (const char*)(sw_config_ptr+ver_ofs[1]), 8) >= 0) {
+				sw_config_ptr = sw_config;
+				DEBUGP(DNM, "Selecting first SW. Unit may reboot now, don't worry.\n");
+			} else {
+				DEBUGP(DNM, "Selecting second SW. Unit may reboot now, don't worry.\n");
+				sw_descr_len_act = sw_descr_len[1];
+			}
+		}
+	}
 
 	return ipacc_sw_activate(sign_link->trx->bts, foh->obj_class,
 				 foh->obj_inst.bts_nr,
 				 foh->obj_inst.trx_nr,
 				 foh->obj_inst.ts_nr,
-				 sw_config, sw_descr_len);
+				 sw_config_ptr, sw_descr_len_act);
 }
 
 /* Receive a CHANGE_ADM_STATE_ACK, parse the TLV and update local state */
